

function_test_power_law <- function(Muestra) {
  
  # Function that obtains the best fit of a sample to a Power - Law and its p-value
  #
  # Input: The sample cointained in a vector
  # Output: Main parameters to be considered and plots of the sample and its fit
  
  ITERACIONES = 100
  PLOT_TYPE = 1
  MIN_N_TAIL = 40
  
  Fpareto <- function(mdat, p) {
    1 - (mdat / p[2])^(-p[1]) }
  
  Fempirica <- function(position, n_length ) {
    position / (n_length + 1) }
  
  Muestra <- sort(Muestra, decreasing = FALSE)
  n <- length (Muestra)
  pos = 0         # It indicates the position where KS is minimized
  pvalorfinal = 0 # Value by default
  
  
  X_min_test <- c(1 : (n - MIN_N_TAIL)) * 0
  alpha_test <- c(1 : (n-MIN_N_TAIL)) * 0
  KS_test <- c(1 : (n-MIN_N_TAIL)) * 0
  
  for(l in 1 : (n-MIN_N_TAIL)) {
    
    Resample = Muestra[l : n] 
    posicion <- rank(Resample, ties.method = "max") 
    X_min_test[l] = Resample[1]
    alpha_test[l] = (n - l) / sum(log(Resample / X_min_test[l]))
    p <- c(alpha_test[l], X_min_test[l])
    
    KS_test[l] = max(abs(Fempirica(posicion, length(Resample)) - Fpareto(Resample, p)))
    
  }
  
  pos = which.min(KS_test)
  alpha = alpha_test[pos]
  sigma = X_min_test[pos]
  KS = min(KS_test)
  
  
  #-----------------------      SEMI-PARAMETRIC BOOTSTRAP      ---------------------------------
  #
  #   Procedure:
  #
  #     1- Generate "m" synthetic datasets as follows:
  #           Take random x from 0 to 1
  #             If x>pos/n -> We generate data from power law
  #             If x<pos/n -> Non-parametric Bootstrap
  #     2- Calculate new alpha/sigmas for each data set
  #     3- Obtain KS for all data sets
  #     4- Obtain p-value and check if p>0.1 (if benevolent 0.05)
  #
  #     5*- If p-value not found, check biyection method with less data 
  #
  #   Notes:
  #   1- Parametric generation of data:
  #         Inverse procedure -> Cuantil (Faustino's procedure)
  #         Rejection method -> Similar to MonteCarlo: not done here, too costly
  #   2- Code efficiency
  #         There is work to be done related to reduction of loops
  #  
  #
  
  
  pvalor <- 0
  
  for (i in 1 : ITERACIONES) {
    
    Samplesim <- c(1 : n) * 0
    
    for(l in 1 : n) {
      if(runif(1) > (pos / n)) { 
        Samplesim[l] = sigma * (1 - runif(1))^(-1 / alpha) }
      else { 
        Samplesim[l] = runif(1) * sigma
      } 
    }
    
    Samplesim <- sort(Samplesim, decreasing = FALSE)
    
    X_min_test2 <- c(1 : (n - 40)) * 0
    alpha_test2 <- c(1 : (n - 40)) * 0
    KS_test2 <- c(1 : (n - 40)) * 0
    
    for(q in 1 : (n - 40)) {
      Resample = Samplesim[q : n] 
      posicion <- rank(Resample, ties.method = "max") 
      X_min_test2[q] = Resample[1]
      alpha_test2[q] = (n - q) / sum(log(Resample / X_min_test2[q]))
      p <- c(alpha_test2[q], X_min_test2[q])
      
      KS_test2[q] = max(abs(Fempirica(posicion, length(Resample)) - Fpareto(Resample, p)))
    }
    
    if(min(KS_test2) > KS) { pvalor <- pvalor + 1 }
    
  }
  
  pvalorfinal <- pvalor / ITERACIONES
  
  
  #------------------------      NON-PARAMETRIC BOOTSTRAP      ---------------------------------
  #
  #   We are willing to obtain the standard deviation of X_min and alpha
  #     Note: if p-value not sufficient - SKIP
  #
  #   Procedure:
  #     1- Resample data with sustitution
  #     2- Using phase 1, get alpha and X_min
  #     3- Repeat i-times and obtain deviation
  #
  

  Set_sigma <- c(1 : ITERACIONES) * 0
  Set_alpha <- c(1 : ITERACIONES) * 0
  
  for(i in 1 : ITERACIONES) {
    
    Resample2 = Muestra[ceiling(runif(n) * n)]
    
    Resample2 <- sort(Resample2, decreasing = FALSE)
    pos2 <- rank(Resample2, ties.method = "max") 
    
    temp_sigma <- c(1 : (n - 40))
    temp_alpha <- c(1 : (n - 40))
    temp_KS <- c(1 : (n - 40))
    
    for(l in 1 : (n - 40)) {
      Resample3 = Resample2[l : n]
      posicion <- rank(Resample3, ties.method = "max") 
      temp_sigma[l] = Resample3[1]
      temp_alpha[l] = (n - l) / sum(log(Resample3 / temp_sigma[l]))
      p <- c(temp_alpha[l], temp_sigma[l])
      
      temp_KS[l] = max(abs(Fempirica(posicion, length(Resample3)) - Fpareto(Resample3, p)))
    }
    
    temp_p = which.min(temp_KS)
    Set_sigma[i] = temp_sigma[temp_p]
    Set_alpha[i] = temp_alpha[temp_p]
    
  }
  
  #-------------------------- RESULTS -------------------------------------
  
  message("----------- FINAL RESULTS: --------------")
  
  message("n: ", length(Muestra))
  
  message("<x>: ", mean(Muestra))
  message("std dev: ", sd(Muestra))
  message("x_max: ", Muestra[length(Muestra)])
  
  message("sigma: ", sigma, "+-", sd(Set_sigma))
  message("alpha: ", alpha, "+-", sd(Set_alpha))
  message("n_tail: ", n-pos)
  message("KS: ", KS)
  
  
  message("p-valor: ", pvalorfinal)
  
  message("-----------------------------------------")  
  
  
  rango = rev(rank(Muestra, ties.method = "first")) 
  tamao = Muestra
  
  if (PLOT_TYPE == 1) {
    
      par(mar=c(5,6,4,1)+.1)
      plot(log(tamao), log(rango), pch = 1, col = "darkblue", xlab = expression(log(tamao)), ylab = expression(log(rango)), cex.lab = 2.)
      lines(log(tamao), log(n - pos + 1) + alpha * log(sigma) - alpha * log(tamao), col = "darkred")
      abline(v = log(sigma), col = "red", lty = 2)
      
  }else if(PLOT_TYPE == 2) {
      
      par(mar=c(5,6,4,1)+.1)
      par(mfrow = c(1, 2))
      plot(log(tamao), log(rango), pch = 1, col = "darkblue", xlab = expression(log(tamao)), ylab = expression(log(rango)), cex.lab = 2.)
      lines(log(tamao), log(n - pos + 1) + alpha * log(sigma) - alpha * log(tamao), col = "darkred")  
      abline(v = log(sigma), col = "red", lty = 2)
      plot(X_min_test, alpha_test, xlab = expression(paste(sigma)), ylab = expression(paste(alpha)), cex.lab = 2.)
      abline(v = sigma, col = "red", lty = 2)
      
  }else if(PLOT_TYPE == 3){
    
      par(mar=c(5,6,4,1)+.1)  
      par(mfrow = c(1, 2)) 
      plot(log(tamao), log(rango), pch = 1, col = "darkblue", xlab = expression(log(tamao)), ylab = expression(log(rango)), cex.lab = 2.)
      lines(log(tamao), log(n - pos + 1) + alpha * log(sigma) - alpha * log(tamao), col = "darkred")  
      size = Muestra[pos : length(Muestra)]
      rank = rev(rank(size, ties.method="first")) 
      plot(log(size), log(rank), pch = 1, col = "darkblue", xlab = expression(log(tamao)), ylab = expression(log(rango)), cex.lab = 2.)
      lines(log(size), log(n - pos + 1) + alpha * log(sigma) - alpha * log(size), col = "darkred")  
  
  }
  
}













